package com.qmth.cqb.paper.service.export;

import java.util.ArrayList;
import java.util.List;

import javax.servlet.http.HttpServletResponse;

import org.springframework.stereotype.Service;

import cn.com.qmth.examcloud.common.dto.core.enums.CourseLevel;
import cn.com.qmth.examcloud.common.dto.question.enums.QuesStructType;
import cn.com.qmth.examcloud.common.uac.entity.AccessUser;

import com.qmth.cqb.paper.dto.PaperDetailExp;
import com.qmth.cqb.paper.dto.PaperDetailUnitExp;
import com.qmth.cqb.paper.dto.PaperExp;
import com.qmth.cqb.paper.model.ExportStructure;
import com.qmth.cqb.paper.model.ExtractConfig;
import com.qmth.cqb.paper.model.Paper;
import com.qmth.cqb.paper.model.PaperDetail;
import com.qmth.cqb.paper.model.PaperDetailUnit;
import com.qmth.cqb.paper.model.QuestionTypeNum;
import com.qmth.cqb.question.model.Question;
import com.qmth.cqb.utils.BeanCopierUtil;
import com.qmth.cqb.utils.CommonUtils;
import com.qmth.cqb.utils.enums.ExamFileType;
import com.qmth.cqb.utils.enums.ExportType;
import com.qmth.cqb.utils.exception.PaperException;
import com.qmth.cqb.utils.word.DocxProcessUtil;

/**
 * 
 * @author  	chenken
 * @date    	2017年7月31日 上午9:21:51
 * @company 	QMTH
 * @description 电子科技大学导出、上传文件service
 */
@Service("dzkdExportPaperService")
public class DzkdExportPaperService extends ExportPaperAbstractService{
	
    /**
     * 下载试卷
     * @param id
     */
    @Override
    public void downloadPaper(String id, HttpServletResponse response) throws Exception {
    	PaperExp paperExp = initExportPaper(id);
        List<String> fileList = new ArrayList<String>();
        if (paperExp!=null) {
        	String paperfileName = paperExp.getName()+DOCX_SUFFIX;
            String answerFileName = paperExp.getName()+"_"+ExamFileType.ANSWER.getName()+DOCX_SUFFIX;
            DocxProcessUtil.exportWord(paperExp,paperfileName,DZKD_TEMPLATE_PAPER);
            DocxProcessUtil.exportWord(paperExp,answerFileName,DZKD_TEMPLATE_ANSWER);
            DocxProcessUtil.processImage(paperfileName,getPkgList(id));
            DocxProcessUtil.processImage(answerFileName,getPkgList(id));
            fileList.add(paperfileName);
            fileList.add(answerFileName);
            DocxProcessUtil.processDownload(fileList, response);
        }
    }
    
	/**
	 * 初始化导出试卷DTO
	 * @param id
	 * @return
	 */
    @Override
    public PaperExp initExportPaper(String paperId) throws Exception {
		// 获取paper
		Paper paper = paperRepo.findOne(paperId);
		paperService.formatPaper(paper, null);
		if (paper == null) {
			throw new PaperException("该试卷不存在");
		}
		// 创建paperDto
		PaperExp paperExp = BeanCopierUtil.copyProperties(paper, PaperExp.class);
		paperExp.setCourseLevel(paper.getLevel()==null?CourseLevel.ALL.name():paper.getLevel().name());
		List<PaperDetailExp> objectiveDetails = new ArrayList<PaperDetailExp>();// 客观题
		List<PaperDetailExp> subjectiveDetails = new ArrayList<PaperDetailExp>();// 主观题
		double objectiveScore = 0;
		double subjectiveScore = 0;
		paperExp.setTitle(CommonUtils.PAPER_TITLE);
		paperExp.setSubTitle(CommonUtils.PAPER_SUB_TITLE);
		// 获取大题
		List<PaperDetail> paperDetails = paperDetailRepo.findByPaper(paper);
		int objectNumber = 1;// 客观题大题序号
		int objetcUnitSum = 0;
		for (int i = 0; i < paperDetails.size(); i++) {
			PaperDetailExp paperDetailExp = BeanCopierUtil.copyProperties(paperDetails.get(i), PaperDetailExp.class);
			List<PaperDetailUnit> paperDetailUnits = paperDetailUnitRepo.findByPaperDetailOrderByNumber(paperDetails.get(i));
			// 把大题分类 ：客观题和主观题
			List<PaperDetailUnitExp> paperDetailUnitExps = BeanCopierUtil.copyPropertiesOfList(paperDetailUnits,PaperDetailUnitExp.class);
			// 选择题，套题下选择题 选项顺序重新排列
			paperService.reorderChoicequestionOption(paperDetailUnitExps);
			paperDetailExp.setPaperDetailUnits(paperDetailUnitExps);
			boolean hasTextAnswer = false;// 判断主观题
			if (paperDetailUnits.get(0).getQuestionType() == QuesStructType.NESTED_ANSWER_QUESTION) {
				for (Question ques : paperDetailUnits.get(0).getQuestion().getSubQuestions()) {
					if (ques.getQuestionType() == QuesStructType.TEXT_ANSWER_QUESTION) {
						hasTextAnswer = true;
						break;
					}
				}
			} else if (paperDetailUnits.get(0).getQuestionType() == QuesStructType.TEXT_ANSWER_QUESTION) {
				hasTextAnswer = true;
			}
			if (hasTextAnswer) {
				subjectiveDetails.add(paperDetailExp);
				subjectiveScore += paperDetailExp.getScore();
			} else {
				objectiveDetails.add(paperDetailExp);
				paperDetailExp.setNumber(objectNumber);
				paperDetailExp.setCnNum(CommonUtils.toCHNum(objectNumber));
				objectiveScore += paperDetailExp.getScore();
				objetcUnitSum += paperDetailExp.getUnitCount();
				objectNumber++;
			}
			setExpTitle(paperDetailExp);
		}

		paperExp.setObjectiveDetails(objectiveDetails);
		paperExp.setSubjectiveDetails(subjectiveDetails);
		paperExp.setObjectiveScore(objectiveScore);
		paperExp.setSubjectiveScore(subjectiveScore);
		if (objectiveDetails.size() > 0) {
			setUnitExpNumber(objectiveDetails, 0);
			appendScoreToQuestionBody(objectiveDetails);
		}
		if (subjectiveDetails.size() > 0) {
			for (PaperDetailExp subDetail : subjectiveDetails) {
				subDetail.setNumber(objectNumber);
				subDetail.setCnNum(CommonUtils.toCHNum(objectNumber));
				objectNumber++;
			}
			setUnitExpNumber(subjectiveDetails, objetcUnitSum);
			appendScoreToQuestionBody(subjectiveDetails);
		}
		return paperExp;
	}

	@Override
    public void uploadFile(ExtractConfig extractConfig,String paperId,ExportStructure exportStructure,AccessUser accessUser) throws Exception{
    	PaperExp paperExp = initExportPaper(paperId);
        if (paperExp!=null) {
        	String currNum = CommonUtils.getCurNum();
        	List<PaperDetailExp> objectiveDetails = paperExp.getObjectiveDetails();
        	//没有试卷结构导出设置
        	if(exportStructure==null){
        		//上传试卷
        		uploadPaperOrAnswerFile(paperExp,extractConfig,accessUser,currNum,DZKD_TEMPLATE_PAPER,ExamFileType.PAPER);
	        	//上传答案
        		uploadPaperOrAnswerFile(paperExp,extractConfig,accessUser,currNum,DZKD_TEMPLATE_ANSWER,ExamFileType.ANSWER);
        	}else if(exportStructure!=null&&exportStructure.getExportType()==ExportType.NORMAL){
        		List<QuestionTypeNum> questionTypeNums = exportStructure.getQuestionTypeNums();
        		//检查试卷中客观题的数量是否大于试卷导出设置中的设置的数量
        		checkObjectiveDetailsNum(paperExp,objectiveDetails,questionTypeNums);
	    		//上传试卷
        		uploadPaperOrAnswerFile(paperExp,extractConfig,accessUser,currNum,DZKD_TEMPLATE_PAPER,ExamFileType.PAPER);
	        	//上传答案
        		uploadPaperOrAnswerFile(paperExp,extractConfig,accessUser,currNum,DZKD_TEMPLATE_ANSWER,ExamFileType.ANSWER);
				//上传试卷结构   不能在上传试卷和答案之前
        		uploadPaperStructure(paperExp,extractConfig,accessUser,currNum,questionTypeNums);
        	}else if(exportStructure!=null&&exportStructure.getExportType()==ExportType.ONLINE){
        		//上传机考JSON文件
        		uploadComputerTestFile(extractConfig,accessUser);
        	}
        }
    }

}
