package com.qmth.cqb.paper.service.export;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;

import javax.servlet.http.HttpServletResponse;

import org.springframework.stereotype.Service;

import cn.com.qmth.examcloud.common.dto.question.enums.QuesStructType;
import cn.com.qmth.examcloud.common.uac.entity.AccessUser;

import com.qmth.cqb.paper.dto.PaperDetailExp;
import com.qmth.cqb.paper.dto.PaperExp;
import com.qmth.cqb.paper.model.ExportStructure;
import com.qmth.cqb.paper.model.ExtractConfig;
import com.qmth.cqb.paper.model.QuestionTypeNum;
import com.qmth.cqb.utils.CommonUtils;
import com.qmth.cqb.utils.enums.ExamFileType;
import com.qmth.cqb.utils.enums.ExportType;
import com.qmth.cqb.utils.word.DocxProcessUtil;

/**
 * @author  	chenken
 * @date    	2017年7月7日 上午11:29:49
 * @company 	QMTH
 * @description 陕西师范大学导出、上传文件service
 */
@Service("sxsfExportPaperService")
public class SxsfExportPaperService extends ExportPaperAbstractService {
	
	@Override
    public void downloadPaper(String paperId, HttpServletResponse response) throws Exception {
    	PaperExp paperExp = initExportPaper(paperId);
        if (paperExp!=null) {
        	List<String> fileList = new ArrayList<String>();
            String paperfileName = paperExp.getName()+DOCX_SUFFIX;
            String answerFileName = paperExp.getName()+"_"+ExamFileType.ANSWER.getName()+DOCX_SUFFIX;
            DocxProcessUtil.exportWord(paperExp, paperfileName,SXSF_TEMPLATE_PAPER);
            DocxProcessUtil.exportWord(paperExp,answerFileName,SXSF_TEMPLATE_ANSWER);
            DocxProcessUtil.processImage(paperfileName,getPkgList(paperId));
            DocxProcessUtil.processImage(answerFileName,getPkgList(paperId));
            fileList.add(paperfileName);
            fileList.add(answerFileName);
            DocxProcessUtil.processDownload(fileList, response);
        }
    }

	@Override
    public void uploadFile(ExtractConfig extractConfig,String paperId,ExportStructure exportStructure,AccessUser accessUser) throws Exception{
    	PaperExp paperExp = initExportPaper(paperId);
        if (paperExp!=null) {
        	String currNum = CommonUtils.getCurNum();
        	List<PaperDetailExp> objectiveDetails = paperExp.getObjectiveDetails();
        	//没有试卷结构导出设置
        	if(exportStructure==null){
        		//上传试卷
        		uploadPaperOrAnswerFile(paperExp,extractConfig,accessUser,currNum,SXSF_TEMPLATE_PAPER,ExamFileType.PAPER);
	        	//上传答案
        		uploadPaperOrAnswerFile(paperExp,extractConfig,accessUser,currNum,SXSF_TEMPLATE_ANSWER,ExamFileType.ANSWER);
        	}else if(exportStructure!=null&&exportStructure.getExportType()==ExportType.NORMAL){
        		List<QuestionTypeNum> questionTypeNums = exportStructure.getQuestionTypeNums();
        		//检查试卷中客观题的数量是否大于试卷导出设置中的设置的数量
        		checkObjectiveDetailsNum(paperExp,objectiveDetails,questionTypeNums);
	    		//上传试卷
        		uploadPaperOrAnswerFile(paperExp,extractConfig,accessUser,currNum,SXSF_TEMPLATE_PAPER,ExamFileType.PAPER);
	        	//上传答案
        		uploadPaperOrAnswerFile(paperExp,extractConfig,accessUser,currNum,SXSF_TEMPLATE_ANSWER,ExamFileType.ANSWER);
				//上传试卷结构   不能在上传试卷和答案之前
        		uploadPaperStructure(paperExp,extractConfig,accessUser,currNum,questionTypeNums);
        	}else if(exportStructure!=null&&exportStructure.getExportType()==ExportType.ONLINE){
        		//上传机考JSON文件
        		uploadComputerTestFile(extractConfig,accessUser);
        	}
        }
    }
    
	@Override
	public void setExpTitle(PaperDetailExp paperDetailExp) {
		String title = "";
		String totalScore = BigDecimal.valueOf(paperDetailExp.getScore()).stripTrailingZeros().toPlainString();
		QuesStructType type = paperDetailExp.getPaperDetailUnits().get(0).getQuestionType();
		String questionScore = checkPaperDetailUnitScore(paperDetailExp);
		String scoreString = questionScore==null?"":"每小题"+questionScore+"分";
		if (type == QuesStructType.SINGLE_ANSWER_QUESTION) {
			paperDetailExp.setName("单项选择题");
			title = "（本大题共" + paperDetailExp.getUnitCount() + "小题，"
					+ scoreString + "共" + totalScore + "分）。"
					+ "在每小题列出的四个备选中只有一个符合题目要求的，请将其选出并将“答题卡”的相应代码涂黑，错涂、多涂或未涂均无分";
		} else if (type == QuesStructType.BOOL_ANSWER_QUESTION) {
			paperDetailExp.setName("判断题");
			title = "（本大题共" + paperDetailExp.getUnitCount() + "小题，"+ scoreString + "共" + totalScore+ "分。正确的填涂A、错误填涂B。错涂、多涂或未涂均无分）";
		} else if (type == QuesStructType.FILL_BLANK_QUESTION) {
			paperDetailExp.setName("填空题");
			title = "（本大题共"+paperDetailExp.getUnitCount()+"小题，共"+totalScore + "分）";
		} else {
			title = "（本大题共"+paperDetailExp.getUnitCount()+"小题，"+scoreString+"共"+totalScore+"分）";
		}
		paperDetailExp.setTitle(title);
	}
	
}

