package com.qmth.cqb.paper.service.impl;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import main.java.com.UpYun;

import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.domain.Example;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.mongodb.core.MongoTemplate;
import org.springframework.data.mongodb.core.query.Criteria;
import org.springframework.data.mongodb.core.query.Query;
import org.springframework.data.mongodb.core.query.Update;
import org.springframework.stereotype.Service;

import cn.com.qmth.examcloud.common.dto.em.enums.ExamType;
import cn.com.qmth.examcloud.common.uac.entity.AccessUser;

import com.qmth.cqb.paper.dao.ExamFileRepo;
import com.qmth.cqb.paper.dao.ExportStructureRepo;
import com.qmth.cqb.paper.dao.ExtractConfigRepo;
import com.qmth.cqb.paper.model.ExamFile;
import com.qmth.cqb.paper.model.ExportStructure;
import com.qmth.cqb.paper.model.ExtractConfig;
import com.qmth.cqb.paper.model.QuestionTypeNum;
import com.qmth.cqb.paper.service.ExamFileService;
import com.qmth.cqb.paper.service.ExportStructureService;
import com.qmth.cqb.utils.enums.ExportType;

/**
 * @author  	chenken
 * @date    	2017年7月12日 下午4:26:49
 * @company 	QMTH
 * @description 导出试卷结构设置
 */
@Service("exportStructureService")
public class ExportStructureServiceImpl implements ExportStructureService {

	@Autowired
	private ExportStructureRepo exportStructureRepo;
	
	@Autowired
	private ExamFileRepo examFileRepo;
	
	@Autowired
	private ExamFileService examFileService;
	
	@Autowired
	private ExtractConfigRepo extractConfigRepo;
	
	@Autowired
	private MongoTemplate mongoTemplate;
	
	@Value("${upyun.bucketName}")
	protected String bucketName;
	
	@Value("${upyun.userName}")
	protected String userName;
	
	@Value("${upyun.password}")
	protected String password;
	
	@Override
	public void saveExportStructure(ExportStructure exportStructure,AccessUser user) {
		if(StringUtils.isBlank(exportStructure.getId())){
			exportStructure.setCreateUser(user.getName());
			exportStructure.setCreateTime(new Date());
		}else{
			exportStructure.setUpdateUser(user.getName());
			exportStructure.setUpdateTime(new Date());
		}
		exportStructure.setOrgId(user.getRootOrgId()+"");
		exportStructure.setOrgName(user.getRootOrgName());
		exportStructureRepo.save(exportStructure);
	}

	@Override
	public ExportStructure findByExportStructure(ExportStructure exportStructure) {
		List<ExportStructure> exportStructures = exportStructureRepo.findAll(Example.of(exportStructure));
		if(exportStructures.size()>0){
			return exportStructures.get(0);
		}
		return null;
	}

	@Override
	public Page<ExportStructure> findPageByExportStructure(ExportStructure exportStructure, int curPage, int pageSize) {
		Page<ExportStructure> list = exportStructureRepo.findAll(Example.of(exportStructure), new PageRequest(curPage - 1, pageSize));
		return list;
	}

	@Override
	public ExportStructure findStructureByExamId(String examId) {
		ExportStructure exportStructure = findByExportStructure(new ExportStructure(examId));
		if(exportStructure==null){
			return null;
		}
		if(exportStructure.getExportType()==ExportType.NORMAL){
			List<QuestionTypeNum> questionTypeNums = exportStructure.getQuestionTypeNums();
			for(QuestionTypeNum questionTypeNum:questionTypeNums){
				if(questionTypeNum.getQuantity()==null){
					//结构设置不完整
					throw new RuntimeException("试卷结构导出设置不完整,请核实后重新设置导出规则！");
				}
			}
		}
		return exportStructure;
	}

	@Override
	public List<String> findExportStructureByExamType(String examType) {
		List<ExportStructure> exportStructures = exportStructureRepo.findAll(Example.of(new ExportStructure(ExamType.strToEnum(examType))));
		if(exportStructures.size()>0){
			List<String> list = new ArrayList<String>();
			for(int i=0;i<exportStructures.size();i++){
				list.add(exportStructures.get(i).getExamId());
			}
			return list;
		}
		return null;
	}

	@Override
	public void deleteExportStructure(String id) {
		//首先根据id，查询到  exportStructure 对象
		ExportStructure exportStructure = exportStructureRepo.findOne(id);
		if(exportStructure != null){
			//1.首先删除ExamFile表       根据  examId 和    orgId  查询   ExamFile对象
			List<ExamFile> list = examFileService.findExamFileListByExamFile(new ExamFile(exportStructure.getExamId(),exportStructure.getOrgId()));
			if(list.size()>0){
				//根据 ExamFile 对象查询  filePath ，并删除又拍云上的文件 , 然后在删除表里面的数据
				UpYun upyun = new UpYun(bucketName,userName,password);
				for(int i=0;i<list.size();i++){
					upyun.deleteFile(list.get(i).getFilePath());
				}
				examFileRepo.delete(list);
			}
			//2.然后修改ExtractConfig表      根据 examId 和    orgId
			Query query = new Query();
			query.addCriteria(Criteria.where("examId").is(Long.valueOf(exportStructure.getExamId())));
			query.addCriteria(Criteria.where("orgId").is(exportStructure.getOrgId()));
			Update update = new Update();
			update.set("ifFinish", 0);
			update.unset("finishedPaperIdMap");
			mongoTemplate.updateMulti(query, update, ExtractConfig.class);
			
			exportStructureRepo.delete(exportStructure);
		}
	}
	
	
	
}

